use std::thread;
use reqwest;
use std::time::{Duration, Instant};
use std::sync::mpsc;

use crate::results::*;

pub fn http_stress(url: String, interval: Duration, sleep: Duration, threads: u32) -> std::io::Result<()> {
    
    let mut handles = vec![];
    let (tx, rx) = mpsc::channel();
    
    println!("Threads starting their {:?} run...", interval);

    for _i in 0..=threads {
        let add = url.clone();
        let end_time = Instant::now() + interval;
        let wait = sleep.clone();
        let tx2 = tx.clone();



        handles.push(thread::spawn( move || {
            loop {
                if Instant::now() >= end_time { break; }

                let b4 = Instant::now();
                let resp = reqwest::blocking::get(add.clone());
                let time = Instant::now() - b4;
                match resp {
                    Ok(_) => {
                        let myres = StressResult { time: time, ok: true, rtype: ResType::Http };
                        let _ = tx2.send(myres);
                        //println!("Got an answer for {} in {:?} in thread {}", add, time, i);
                    },
                    Err(_) => {
                        let myres = StressResult { time: time, ok: false, rtype: ResType::Http };
                        let _ = tx2.send(myres);
                        //println!("No answer for {} after {:?} in thread {}", add, time, i);
                    },
                }
                thread::sleep(wait);
            }
            drop(tx2);
        }));
    }
    drop(tx);

    for handle in handles {
        let _ = handle.join();
    }
    println!("over !");

    //Showing Results
    //Somehow iterating over never ends
    let mut res: Vec<StressResult> = vec![];
    for r in rx.try_iter() {
        res.push(r);
    }
    let lres = ListOfResults { results: res, restype: ResType::Http };
    lres.show_info();
    Ok(())
}
                




